; Interface and utility functions

(defun help () "Help message" (printc help) (values))

(setq help
"
Useful functions:
Installing forms  - INST INSTALL QINST QINSTALL
Defining notions  - DEFINE DEFNTN
Reducing forms    - HIT QHIT PPHIT WHIT
Garbage collector - HGC QGC
Array status      - REPORT
Print forms       - WFF PPWFF
Statistics        - STATREPORT

(DOC '<fn>' ...) prints info about functions.

EXAMPLES is a list of current examples.

"
)

(defvar examples nil)
(defvar test nil)

(defun init () (*cold-boot) (setup)
  (setq FIRINGS (setq TOTRDTNS 0)) (make-graph test))

(defmacro inst (expr)
"(inst 'expr') -- install expr for testing"
 `(install ',expr)
)

(defun install (expr)
"(install expr) -- function like install"
 (installx expr)  (report) (format t "Root is ~D~%" ROOT)
 (values)
)

(defmacro qinst (expr) `(qinstall ',expr))

(defun qinstall (expr) 
 (installx expr) (format t "Root is ~D~%" ROOT)
 (values)
)

(defun installx (expr)
 (*set STATS (!! 0)) (setq FIRINGS (setq TOTRDTNS 0))
 (setq test expr) (make-graph test)
)

(defun hitx (n)
  (dotimes (k n) (fire)
     (when REDERROR (format t "~%Reduction error:  ~A~% REDERROR")
	            (return-from hitx 'ERROR)))
)

(defun hit (&optional (n 1))
"(hit { n } ) -- FIRE { n times }, report"
 (hitx n) (report)
)

(defun pphit (&optional (n 1))
"(pphit { n } ) -- FIRE { n times }, pprint root"
 (hitx n) (ppwff root)
)

(defun whit (&optional (n 1))
"(whit { n } ) -- FIRE { n times }, returns (wff root)"
 (hitx n) (wff root)
)

(defun qhit (&optional (n 1))
"(qhit { n } ) -- FIRE { n times } , say reductions"
 (let ((r0 TOTRDTNS))
   (hitx n) (format t "~D reductions" (- TOTRDTNS r0)))
 (values)
)

(defun qgc (&optional (n GBGCYCLE))
"(gc { n } ) -- gc { n times }"
 (let ((g nil))
    (sRN!! (!! 255)) (setq g (garbage n))
    (when g (*when (<!! RDTN (!! 255))
		 (setq REDUCTIONS (*sum (!! 1))) (gatherstats))
            (incf TOTRDTNS REDUCTIONS))
  g)
)

(defun hgc (&optional (n GBGCYCLE))
"(hgc { n } ) -- gc { n times } with report"
 (if (qgc n) (report) (format t "No garbage~%"))
 (values)
)

(defun ppwff (n)
"(ppwff node) -- pretty-prints formula at node"
 (pprint (wff n))
)